import javax.sql.rowset.RowSetProvider;
import javax.sql.rowset.RowSetFactory;
import javax.sql.rowset.CachedRowSet;
import javax.sql.RowSetMetaData;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import com.mysql.jdbc.jdbc2.optional.MysqlDataSource;

public class RowsetMetadataExample {

	private final static String DB_URL = "jdbc:mysql://localhost:3306/example_db";
	private final static String USR = "root";
	private final static String PWD = "root";
	private final static String BOOKS_TABLE = "books_table";
	private final static String NEW_COLUMN_NAME_1 = "book_title";
	private final static String NEW_COLUMN_NAME_2 = "book_author";

	public static void main(String [] args)
			throws Exception {

		CachedRowSet crs = getCachedRowset();
		Connection con = getDataSourceConnection();
		populateRowset(crs, con);
		RowSetMetaData md = getMetaData(crs);
		setMetaData(crs, md);
		crs.close();
		System.out.println("Close.");
	}

	private static CachedRowSet getCachedRowset()
			throws SQLException {

		System.out.println("Create cached rowset");
		RowSetFactory rsFactory = RowSetProvider.newFactory();
		CachedRowSet rowset = rsFactory.createCachedRowSet();
		return rowset;
	}

	private static Connection getDataSourceConnection()
			throws SQLException {

		System.out.println("Get connection to database: " + DB_URL);
		MysqlDataSource myDs = new MysqlDataSource();
		myDs.setURL(DB_URL);
		DataSource ds = myDs;
		Connection con = ds.getConnection(USR, PWD);
		return con;
	}

	private static void populateRowset(CachedRowSet crs, Connection con)
			throws SQLException {

		System.out.println("Populate rowset with database table rows: " + BOOKS_TABLE);
		String sql = "SELECT * FROM " + BOOKS_TABLE;
		crs.setCommand(sql);
		crs.execute(con);
		crs.last();
		System.out.println("Total rows in table: " + crs.getRow());
	}

	private static RowSetMetaData getMetaData(CachedRowSet crs)
			throws SQLException {

		RowSetMetaData metaData = (RowSetMetaData) crs.getMetaData();

		System.out.println("Row set meta data: ");
		int cols = metaData.getColumnCount();
		System.out.println("  Column count: " + cols);

		for (int i = 1; i <= cols; i++) {

			System.out.println("  Column: " + i);
			System.out.println("    Table: " + metaData.getTableName(i));
			System.out.println("    Column name: " + metaData.getColumnName(i));
			System.out.println("    Column type: " + metaData.getColumnTypeName(i));
			System.out.println("    Column is nullable (0=No Nulls, 1=Nullable): " + metaData.isNullable(i));
		}

		return metaData;
	}

	private static void setMetaData(CachedRowSet crs, RowSetMetaData metaData)
			throws SQLException {

		metaData.setColumnName(1, NEW_COLUMN_NAME_1);
		metaData.setColumnName(2, NEW_COLUMN_NAME_2);
		crs.setMetaData(metaData);

		metaData = (RowSetMetaData) crs.getMetaData();
		System.out.println("Row set meta data - with updated column names: ");
		System.out.println("    Column 1 name: " + metaData.getColumnName(1));
		System.out.println("    Column 2 name: " + metaData.getColumnName(2));
	}
}